/*
 * Copyright 2010 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK
// AUTHOR:   Alexander Kamkin (kamkin@ispras.ru)
// SYNOPSIS: Debug printing facilities 
///////////////////////////////////////////////////////////////////////////////

#include <hw/debug.hpp>

namespace cpptesk {
namespace hw {

// By default, only warnings and errors are printed
debug_level_t Debug::_debug_level = DEBUG_WARNING;

const char *Debug::_STRING[] = { "FAIL",        "WARN",        "USER",        "INFO"        };
const char *Debug::_STYLE [] = { "\x1b[30;41m", "\x1b[30;43m", "\x1b[30;46m", "\x1b[30;42m" };

const char *Debug::_NORMAL_STYLE = "\x1b[0m";

void Debug::level(debug_level_t debug_level)
{
    _debug_level = debug_level;
}

void Debug::print(debug_level_t debug_level, const std::string &debug_message)
{
    std::ostream &out = std::cout;

    if((int)_debug_level >= (int)debug_level)
    {
        out << "------------------------------------------------------------" << std::endl;

#ifndef CPPTESK_TERMINAL_HIGHLIGHTING_DISABLE
        out << _STYLE[debug_level];
#endif // CPPTESK_TERMINAL_HIGHLIGHTING_DISABLE

        out << "[" << _STRING[debug_level] << "]";

#ifndef CPPTESK_TERMINAL_HIGHLIGHTING_DISABLE
        out << _NORMAL_STYLE;
#endif // CPPTESK_TERMINAL_HIGHLIGHTING_DISABLE

        out << " ";
        out << debug_message << std::endl;
    }
}

}} // namespace cpptesk::hw

