/*
 * Copyright 2010 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK
// AUTHOR:   Alexander Kamkin (kamkin@ispras.ru)
// SYNOPSIS: Set of reactions 
///////////////////////////////////////////////////////////////////////////////

#pragma once

#include <hw/defs.hpp>

#include <hw/errors.hpp>
#include <hw/reaction.hpp>

#include <string>
#include <vector>

namespace cpptesk {
namespace hw {

enum error_mode_t { LAST_REACTION, LAST_ERROR, ALL_ERRORS  };

class Reactions
{
public:
    const static unsigned HISTORY_SIZE = 100;

    Reactions();
    virtual ~Reactions();

    /// Adds a reaction.
    void registerReaction(const Reaction &reaction);
    /// Removes reactions from the current set and appends them to the history.
    void clearReactions();
    
    // @depricated
    const std::vector<Reaction>& getReactions() const;

    /// Returns true iff there is at least one errornous reaction in the current set.
    /// Removes reactions from the current set and appends them to the history.
    bool verdict();

    /// Returns an error message or a concatenation of several messages.
    std::string error(error_mode_t = LAST_ERROR) const;

    /// Returns human-understandable diagnostics for reactions.
    /// Analyzes the reactions history only (the current set should be empty).
    std::string diagnosis() const;

    /// Returns a number of reactions (errors) of a given type.
    unsigned count(error_type_t error) const;

    /// Removes all reactions including the history and resets statistics.
    void reset();

protected:
    typedef std::vector<Reaction> reactions_list_t;
    /// Current set of reactions
    reactions_list_t _reactions;
    /// History of reactions
    reactions_list_t _history;

    /// Reactions statistics
    unsigned _counts[__ERROR_TYPE_NUMBER];
};

}} // namespace cpptesk::hw

