/*
 * Copyright 2010 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

///////////////////////////////////////////////////////////////////////////////
// PROJECT:  C++TESK
// AUTHOR:   Alexander Kamkin (kamkin@ispras.ru)
// SYNOPSIS: Veritool-based simulator synchronizer 
///////////////////////////////////////////////////////////////////////////////

#pragma once

#include <hw/defs.hpp>

#include <hw/signals.hpp>
#include <hw/synchronizer.hpp>

// Inputs and output structures are generated by Veritool
struct inputs_t;
struct outputs_t;

// The following functions are generated as well
extern void start_testbench(void);
extern void stop_testbench(void);
extern void map_state_down(inputs_t*);
extern void map_state_up(outputs_t*);
extern void clear_inputs(inputs_t*);
extern void clear_outputs(outputs_t*);

// Library functions of Veritool
extern "C" {
    extern void wait_for_check(void);
};

namespace cpptesk {
namespace hw {
namespace veritool {

template <typename I, typename O>
class VeritoolSynchronizer: public ::cpptesk::hw::Synchronizer
{
public:
    typedef I inputs_t;
    typedef O outputs_t;

    VeritoolSynchronizer(::cpptesk::hw::IOSignals<inputs_t, outputs_t> &signals): _signals(signals) {}
   ~VeritoolSynchronizer() {}

    virtual void initialize() { ::start_testbench(); clear_inputs(&_signals.inputs); clear_outputs(&_signals.outputs); } 
    virtual void finalize()   { ::stop_testbench(); }
    virtual void setInputs()  { ::map_state_down(&_signals.inputs); }
    virtual void getOutputs() { ::map_state_up(&_signals.outputs); }
    virtual void simulate()   { ::wait_for_check(); }

private:
    ::cpptesk::hw::IOSignals<inputs_t, outputs_t> &_signals;
};

}}} // namespace cpptesk::hw::veritool

