#!/bin/bash

# This script installs ICARUS Verilog if $ICARUS_HOME is not set,
#                      Veritool if $VERITOOL_HOME is not set,
#             copy cpptesk-toolkit.tar.gz to its dir and unpack it.

this=`basename "$0"`

usage () {
   cat <<EOF
Usage:
  $this [options]

  This script installs CPPTESK Testing ToolKit. Also VeriTool and Icarus Verilog can be installed by demand.

  Options:
    --install-veritool        - install VeriTool and Icarus Verilog if they have not been installed yet
    --force-install-veritool  - install VeriTool and Icarus Verilog anyway
    --version | -v            - show a version
    --help | -h               - show this message
EOF
}

argc=$(( $# ))

for(( i = 0; $i < $argc; i++ ))
do
  case $1 in
    --version | -v )
      echo $this 1.0.5
      echo ""
      echo "Copyright 2010-2011 (c) ISPRAS"
      echo "Institute for System Programming of the Russian Academy of Sciences"
      echo "25 A.Solzhenitsyn st. Moscow 109004 Russia"
      echo "http://hardware.ispras.ru"
      echo ""
      exit 0
    ;;
    --help | -h )
      usage
      exit 0
    ;;
    --force-install-veritool )
      ./bin/install-veritool.sh --force
    ;;
    --install-veritool )
      ./bin/install-veritool.sh
    ;;
    * )
      usage
      exit 0
  esac
  shift
done

if [ "x" == "x$ISPRAS_HOME" ]
then
  ISPRAS_HOME=$HOME
  UPDATE_PROFILE=1
fi

# Paths for C++TESK Testing ToolKit:
#CPPTESK_DIR="$HOME/downloads/sources/cpptesk"

if [ "x" == "x$CPPTESK_HOME" ]
then
  CPPTESK_HOME="$ISPRAS_HOME/tools/cpptesk-toolkit"
  UPDATE_PROFILE_CPPTESK=1
fi

CURRENT_DIR=`pwd`

# Dependences for Icarus Verilog:
#sudo apt-get build-dep verilog
# Dependences for C++TESK Testing ToolKit:
#sudo apt-get install libmicrohttpd-dev

###################
# Cpptesk-ToolKit #
###################
mkdir -p "$CPPTESK_HOME"
cd "$CURRENT_DIR"
make
cp cpptesk-toolkit.tar.gz "$CPPTESK_HOME"/..
cd "$CPPTESK_HOME"/..
tar xf cpptesk-toolkit.tar.gz
rm cpptesk-toolkit.tar.gz

if [ "x" != "x$UPDATE_PROFILE_CPPTESK" ]
then
  echo "export CPPTESK_HOME=$CPPTESK_HOME" >> "$HOME/.profile"
  echo "export CPPTESK_HOME=$CPPTESK_HOME" >> "$HOME/.bashrc"

fi

export ISPRAS_HOME="$ISPRAS_HOME"

if [ "x" != "x$UPDATE_PROFILE" ]
then
  echo "export ISPRAS_HOME=$ISPRAS_HOME" >> "$HOME/.profile"
  echo "export ISPRAS_HOME=$ISPRAS_HOME" >> "$HOME/.bashrc"
fi

cd "$CURRENT_DIR"/bin
./install-vim-syntax.sh

echo "[OK] CPPTESK Testing ToolKit has been successfully installed."

